# -*- coding: utf-8 -*-

import datetime

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.template.loader import render_to_string
from django.utils.http import int_to_base36

from jmb.mailer.models import EmailObject, SMTPServer, BounceServer
from jmb.mailer.utils.mail_utils import body_insertion
from jmb.mailer.sender import Mailer


class Mailer(Mailer):
    def __init__(self, generic_object, test=False, verbose=2, is_newsletter=False, is_newsletter2=False, is_task=False, request=None):
        super(Mailer, self).__init__(generic_object, test, verbose, is_newsletter, is_newsletter2, is_task, request)

    def create_message_from_email(self, string_email, context_add):
        """
        crea un messaggio partendo da un oggetto Newsletter e da una stringa email
        """
        from acomea.contact.models import Email, Contact
        self.string_email = string_email

        try:
            email = Email.objects.get(email__iexact=string_email)
            contact = email.contact
            self.contact = contact
        except Email.DoesNotExist:
            email = contact = None
        except Email.MultipleObjectsReturned:
            email = contact = None
        except Contact.DoesNotExist:
            contact = None

        if contact and not contact.can_create_message[0]:
            return False, _("can not create message")

        try:
            email_object = EmailObject()
            if self.test:
                """ lo stato dell'email e' 'sent test' """
                email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND_IN_TEST
            else:
                """ lo stato dell'email e' 'to send' """
                email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND

            email_object.email_address = string_email
            email_object.subject = self.generic_object.get_default_title()
            email_object.content_object = self.generic_object
            email_object.content = self._get_newsletter_html_output(context_add)
            email_object.slug = datetime.datetime.now().strftime("%Y%m%d_%H%M%S%f")
            email_object.smtp_server = SMTPServer.objects.get()
            email_object.bounce_server = BounceServer.objects.get()
            email_object.save()
            return True, email_object
        except Exception, e:
            return False, e

    def insert_unsuscribe_link(self, content, context):
        # FIXME: se acomea, utilizzato contact_token_generator di acomea.contact.
        # FIXME: Valutare la sostituzione per la conferma privacy di acomea
        # FIXME: con jmb.mailer.utils.tokens.tokenize
        if 'acomea.contact' in settings.INSTALLED_APPS:
            from acomea.contact.utils.tokens import contact_token_generator
            from acomea.contact.models import Email
            try:
                email = Email.objects.get(email=self.email.email_address)
                contact = email.contact
                token = contact_token_generator.make_token(contact)
                cidb36 = int_to_base36(contact.id)
                context.update({
                    'uidb36': cidb36,
                    'token': token,
                })
            except (ObjectDoesNotExist, MultipleObjectsReturned):
                pass
        unsubscribe_link = render_to_string(
            'mailer/email_unsubscribe_link.html', context
        )
        return body_insertion(content, unsubscribe_link, end=True)