# coding: utf-8
"""
.. _check_contact_group:

Check Contact Group
===================

Command to check align "ContactGroup" model with through between "Contact" and "Group".
An email is sent at end of check.

Note ContactGroup is a model to store "date_create" and "date_last_modify"

usage::

  dj check_contact_group

"""


import logging
import datetime
from optparse import make_option

from django.db import connections
from django.conf import settings
from django.core.mail import send_mail
from django.core.management.base import BaseCommand


# LOGGER
logger = logging.getLogger('acomea.contact')


QUERY_RETRIEVE_UNALIGNED = """
SELECT DISTINCT(T1.contact_id, T1.group_id)
FROM contact_contact_groups AS T1 LEFT JOIN public.contact_contactgroup AS T2
    ON T1.group_id = T2.group_id AND T1.contact_id = T2.contact_id
WHERE T2.id IS NULL;
"""


class Command(BaseCommand):

    option_list = BaseCommand.option_list + (
        # debug level in console
        make_option(
            '-l', '--debug-level', default='debug',
            help='Set log level (debug, info, warnings) for file.'
                 '"tail -24lf /var/log/django/www.acomea.it.contact.log" to see',
        ),
    )

    def handle(self, *args, **options):

        log_level = getattr(logging,  options.get('debug_level', 'info').upper())
        logger.setLevel(log_level)
        logger.info('retrieving data..')
        cursor = connections['default'].cursor()
        cursor.execute(QUERY_RETRIEVE_UNALIGNED)
        result = cursor.fetchall()
        result_list = [row[0] for row in result]
        email_text = """
Di seguito il disallineameto fra ContactGroup e la Through Contact->Group:

Query:
{query}

Per ripristinare l'allineamento eseguire il comando "dj sync_contact_group"

RIGHE DA ALLINEARE:
-------------------

{data}
""".format(query=QUERY_RETRIEVE_UNALIGNED, data=str('\n'.join(result_list)) or 'Niente da allineare')
        sent = send_mail(
            'Check contact group {}'.format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")),
            email_text,
            'check_contact_group@acomea.it',
            settings.CHECK_CONTACT_GROUP_REPORT_MAIL_LIST,
            fail_silently=False,
        )
        if sent:
            logger.info('email sent to {}'.format(', '.join(settings.CHECK_CONTACT_GROUP_REPORT_MAIL_LIST)))
        else:
            logger.error('error when sending the email')
