# coding: utf-8

import logging

from django.core.management.base import BaseCommand
from optparse import make_option

from acomea.contact.utils import cleanup_double_contacts


class Command(BaseCommand):
    """
    unisce contatti con email duplicata e rimuove i doppioni
    """
    help = "Unisce contatti con email duplicata e rimuove i doppioni"

    option_list = BaseCommand.option_list + (
        make_option('-e', '--email', dest='email', help='Email to manage', ),
        make_option('-f', '--file', dest='file', default=None, help="manage duplicated from email in file"),
        make_option('-o', '--output-file', dest='output-file', default=None, help="Output file name for email in error"),
        make_option('-d', '--delete', action='store_true', default=False, help="Delete error file"),

        make_option('-c', '--console', action='store_true', default=False, help='Debug - write logging to console'),
        make_option('-l', '--debug-level', default='info', help='Set debug level (debug, info, warnings) for console'),
        make_option('-x', '--no-stop', action='store_true', default=False, help="Don't stop on error, just log"),
    )

    output_transaction = True

    def handle(self, **options):

        mgr = cleanup_double_contacts.CleanupDoubleContacts()

        level = getattr(logging, options.get('debug_level').upper())
        mgr.logger.setLevel(level)

        if options.get('console'):
            console_handler = logging._handlers['console']
            console_handler.setLevel(level)
            mgr.logger.handlers = []
            mgr.logger.addHandler(console_handler)

        mgr.cleanup(options=options)

# NO BLOCK
# dj cleanup_double_contacts -ldebug -c -d -o/tmp/e.txt -x
# NO BLOCK WITH EMAIL
# dj cleanup_double_contacts -ldebug -c -d -o/tmp/e.txt --email xxxx@sdcdefc.it -x
# NO BLOCK WITH INPUT FILE
# dj cleanup_double_contacts -ldebug -c -d --file/tmp/import.txt -o/tmp/e.txt -x
