# -*- coding: utf-8 -*-

import datetime
import logging
from django.conf import settings
from django.utils.timezone import utc

from acomea.mailup.utils.client import MailUpClient


# def contact_group_signal(instance, action, pk_set, **kwargs):
#     """
#     :param instance: signal sender instance. In our case it is an contact instance
#     :param action: parmeter which identifies the action in signal
#     :param pk_set: a list of referenced ids (in our case of groups) involved in signal
#     """
#     from acomea.contact.models import Group, ContactGroup
#     now = datetime.datetime.utcnow().replace(tzinfo=utc)
#     try:
#         instance_to_save = False
#         pk_list = [int(id) for id in pk_set] if pk_set else list()
#
#         if action == "post_add":
#             for group_id in pk_list:
#                 group = Group.objects.get(pk=group_id)
#                 contact_group, created = ContactGroup.objects.get_or_create(
#                     contact=instance,
#                     group=group
#                 )
#                 if created:
#                     instance_to_save = True
#
#             if not instance.first_group:
#                 first_group = instance.get_first_group_from_contact_group()
#                 if first_group:
#                     instance.first_group = first_group
#                     instance_to_save = True
#
#         if instance_to_save:
#             instance.date_last_modify_for_mailup = now
#             instance.save()
#
#     except Exception, e:
#         logger = logging.getLogger('file_contact')
#         logger.error("Exception in contact_group_signal: {}".format(e))


def contact_group_signal(instance, action, pk_set, **kwargs):
    """
    :param instance: signal sender instance. Can be Contact or Group
    :param action: parmeter which identifies the action in signal
    :param pk_set: a list of referenced ids involved in signal
    """
    from acomea.contact.models import Group, Contact, ContactGroup

    try:
        now = datetime.datetime.utcnow().replace(tzinfo=utc)
        is_add_from_contact = isinstance(instance, Contact)

        if action == "post_add":
            if is_add_from_contact:
                # ES: contact.groups.add(g1):
                #  - instance = Contact
                #  - pk_set   = [g1.pk, g2.pk, ...]
                contacts_id = Contact.objects.filter(pk=instance.pk).values_list('pk', flat=True)
                groups_id = pk_set
            else:
                # ES: group.contacts.add(c1):
                #  - instance = Group
                #  - pk_set   = [c1.pk, c2.pk, ...]
                contacts_id = pk_set
                groups_id = Group.objects.filter(pk=instance.pk).values_list('pk', flat=True)

            for group_id in groups_id:
                for contact_id in contacts_id:

                    contact_to_save = False

                    contact_group, created = ContactGroup.objects.get_or_create(
                        contact_id=contact_id,
                        group_id=group_id
                    )
                    contact = Contact.objects.get(pk=contact_id)

                    if created:
                        contact.date_last_modify_for_mailup = now
                        contact_to_save = True

                    if not contact.first_group:
                        first_group = contact.get_first_group_from_contact_group()
                        if first_group:
                            contact.first_group = first_group
                            contact_to_save = True

                    if contact_to_save:
                        contact.save()
    except Exception as e:
        logger = logging.getLogger('acomea.contact')
        logger.error("Exception in contact_group_signal: {}".format(e))


def delete_mailup_recipient(sender, **kwargs):
    log = logging.getLogger("mailup")
    contact = kwargs['instance']
    mailup_client = MailUpClient()
    mailup_client.retrieve_access_token()
    if contact.mailup_recipient_id:
        mailup_client.unsubscribe_recipient(settings.ACOMEA_MAILUP_LIST_ID, contact.mailup_recipient_id)
        log.debug("Contatto {} disiscritto per eliminazione su database".format(contact.get_default_email()))
