#-*- coding: utf-8 -*-

import logging
import time
from celery import task
from django.conf import settings

from acomea.mailup.utils.client import MailUpClient

log = logging.getLogger("mailup")

@task()
def update_contact_mailup(contact):

    list_id = settings.ACOMEA_MAILUP_LIST_ID
    status_dict = {True: "Subscribed", False: "Unsubscribed"}
    is_subscribed, msg = contact.can_create_message
    new_recipient_status = status_dict[is_subscribed]

    mailup_client = MailUpClient()
    mailup_client.retrieve_access_token()

    log.debug("### INIZIO TASK SINCRONIZZAZIONE CONTATTO %s ###" % contact.get_default_email())
    if not contact.mailup_recipient_id:

        # Importo contatto in mailup
        import_id = mailup_client.import_recipient(list_id, contact)
        import_status_json = mailup_client.read_import_status(import_id)
        while not import_status_json['Completed']:
            time.sleep(1)

        # Recupero recipient id da mailup e lo associo al contatto
        for status in ["Subscribed", "Unsubscribed"]:
            check_subscriber_exists_json = mailup_client.check_subscriber_exists(
                list_id, status, contact.get_default_email(),
            )
            contact_exists = True if check_subscriber_exists_json["TotalElementsCount"] else False
            log.debug("Contact created recipient_id: %s" % contact.mailup_recipient_id)
            if contact_exists:
                recipient = check_subscriber_exists_json["Items"][0]
                contact.mailup_recipient_id = recipient["idRecipient"]
                break

    if contact.mailup_recipient_id:
        log.debug("Contact recipient_id: %s" % contact.mailup_recipient_id)
        log.debug("Contact status: %s " % new_recipient_status)
        if new_recipient_status == "Subscribed":
            mailup_client.subscribe_recipient(list_id, contact.mailup_recipient_id)
        elif new_recipient_status == "Unsubscribed":
            mailup_client.unsubscribe_recipient(list_id, contact.mailup_recipient_id)
        log.debug("Contact mailup sync done")