# coding: utf-8

import logging
import os

from django.contrib.auth import models
from codecs import open

from acomea.contact import models
from django.db.models import Count
from django.db.models.signals import post_delete


class CleanupDoubleContacts(object):

    logger = logging.getLogger('acomea.contact')

    def cleanup(self, options):

        from acomea.contact.signals import delete_mailup_recipient
        post_delete.disconnect(
            receiver=delete_mailup_recipient,
            sender=models.Contact,
            weak=False
        )

        email = options.get('email')
        file_input = options.get('file')
        cleaned_ok = 0
        cleaned_ko = 0
        emails = models.Email.objects.all()
        if email:
            emails = emails.filter(email=email)
        if file_input:
            with open(file_input, "rb") as input:
                list_to_filter = [(email.split("\n")[0]) for email in input]
                emails = emails.filter(email__in=list_to_filter)
        try:
            self.logger.info('Start manage emails')
            emails = emails.values('email').annotate(email_count=Count('email')).filter(email_count__gt=1)

            error_file_name = options.get('output-file') or "/tmp/cleaned_acomea_contact.txt"

            if options.get('delete') and os.path.exists(error_file_name):
                os.remove(error_file_name)

            with open(error_file_name, "a", 'utf-8') as error_file:
                count = len(emails)
                for obj in emails:
                    email_value = obj['email']
                    self.logger.debug(u"Try {}...({})\n".format(email_value, count))
                    try:
                        contacts = models.Contact.objects.filter(
                            emails__email=email_value).order_by('-date_last_modify')

                        try:
                            contact_main = contacts.get(user__isnull=False)
                        except models.Contact.DoesNotExist:
                            contact_main = contacts[0]

                        contact_with_privacy = contacts[0]

                        for contact in contacts:
                            if contact.pk == contact_main.pk:
                                continue
                            for group in contact.groups.all():
                                contact_main.groups.add(group)

                        if contact_main.pk != contact_with_privacy.pk:
                            contact_main.auth_trattamento_dati = contact_with_privacy.auth_trattamento_dati
                            contact_main.auth_materiale_commerciale = contact_with_privacy.auth_materiale_commerciale
                            contact_main.auth_societa_terze = contact_with_privacy.auth_societa_terze
                            contact_main.auth_premium = contact_with_privacy.auth_premium
                            contact_main.mailup_recipient_id = contact_with_privacy.mailup_recipient_id
                            contact_main.save()

                        contacts_to_delete_list = contacts.exclude(pk=contact_main.pk).values_list('pk', flat=True)
                        self.logger.info(
                            u'CLEANED %s, save contact %s, delete %r',
                            email_value, contact_main.pk, contacts_to_delete_list
                        )
                        contacts.exclude(pk=contact_main.pk).delete()

                        cleaned_ok += 1

                    except Exception as e:
                        self.logger.error(u"NOT CLEANED : Email: {}, {}\n".format(email_value, e))
                        msg = u"NOT CLEANED;{};\n".format(email_value)
                        error_file.write(msg)
                        if options.get('no_stop'):
                            pass
                        else:
                            raise
                        cleaned_ko += 1

                    count -= 1
        except Exception as e:
            self.logger.fatal(e)
            raise
        self.logger.info(u"CLEANED: {}\t NOT CLEANED: {}".format(cleaned_ok, cleaned_ko))
