#-*- coding: utf-8 -*-

import logging

from django.contrib import messages
from django.conf import settings
from django.template import loader
from django.template.base import Context
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from jmb.mailer.sender import Mailer
from jmb.mailer.models import SMTPServer
from acomea.event.models import Invite

logger = logging.getLogger('celery.task')

def get_queryset(obj, request, queryset):
    if request.POST.get('select_across') == '1':
        # se sono stati selezionati tutti i contatti (anche delle pagine successive)
        # quindi occorre ricostruire il queryset
        queryset = obj.queryset(request)
    return queryset


def render_template_from_string(context_dict, body, add_contact_querystring=False):
    if 'invite' in context_dict:
        if add_contact_querystring:
            context_dict['invite'].get_landingpage = mark_safe(context_dict['invite'].get_landingpage_with_contact_flag_text_url())
            context_dict['invite'].get_page_url = mark_safe(context_dict['invite'].get_page_url()+"?firstAccess=true")
        else:
            context_dict['invite'].get_landingpage = mark_safe(context_dict['invite'].get_landingpage_text_url())
    context = Context(context_dict)
    template = loader.get_template_from_string(body)
    ret = template.render(context)
    return ret if not "<code></code>" in ret else ret.replace("<code></code>", "")


def print_messages(request, event, sent_in_test, create, not_create, test):
    # PRINT MESSAGES
    if create == 0 and not test:
        messages.error(request, _('%s not create') % event)
    if sent_in_test == 0 and test:
        messages.error(request, _('%s not sent in test') % event)
    if create > 0 and not test:
        messages.info(request, _('{0} succesfully create to {1} contacts').format(
            event, create)
        )
    if sent_in_test > 0 and test:
        messages.info(request, _('{0} succesfully sent in test to {1} contacts').format(
            event, sent_in_test)
        )
    if not_create > 0:
        messages.error(request, _('{0}: {1} in error').format(event, not_create))
    return


def send_invite_to_contact(
        contact, event, test=False, is_task=True, request=None,
        is_trace_request=False, continue_if_not_created=False,
        test_in_subject=True, check_can_create=True,
        add_contact_querystring=False, date_from=None, date_to=None
):
    logger.info("args: %r" % locals())


    sent_in_test = False
    created = False
    not_created = False

    if check_can_create:
        if not is_trace_request:
            if not contact.can_create_message[0]:
                return dict(sent_in_test=sent_in_test, created=created, not_created=not_created)
        else:
            if not contact.can_create_message_no_bonifica[0]:
                return dict(sent_in_test=sent_in_test, created=created, not_created=not_created)

    invite, invite_created = Invite.objects.get_or_create(event=event, contact=contact)

    # Update is_trace_request flag field if invite is create and invite is made from trace request plugin
    if invite_created and is_trace_request:
        invite.is_trace_request = True
        invite.save()
    
    # Send mail if invite is created or continue_if_not_created is True
    if invite_created or continue_if_not_created:
        adhesion = event.translations.get(
            language__id=event.default_language.id,
        )
        mailer = Mailer(
            invite, test, is_task=is_task, request=request,
            is_trace_request=is_trace_request, test_in_subject=test_in_subject,
            email_from=adhesion.email_from
        )
        description = render_template_from_string(
            dict(contact=contact, invite=invite, is_email=True),
            adhesion.body,
            add_contact_querystring
        )
        context = dict(
            description=description,
            invite=invite,
        )
        if adhesion.body_image:
            context["top_image_url"] = "%s://%s/%s" % (
                settings.PROFILO_TCP_PROTOCOL,
                settings.PROFILO_HOST,
                adhesion.body_image.url.lstrip("/")
            )

        mail_created, email_object = mailer.create_message(
            contact, context, check_can_create=check_can_create,
            date_from=date_from if not test else None,
            date_to=date_to if not test else None
        )
        if mail_created:
            created = True
            if test or is_trace_request:
                mailer.send(email_object)
                invite.sent = True
                invite.save()
            if test:
                sent_in_test = True

        else:
            not_created = True
    info = {
        'sent_in_test': sent_in_test,
        'created': created,
        'not_created': not_created,
        'invite_created': invite_created,
        }
    return info


def test_smtp_connection(request):
    try:
        smtpserver = SMTPServer.objects.get()
        smtpserver.connect()
    except:
        messages.error(request, _('SMTP Authentication failed'))
