# -*- coding: utf-8 -*-
"""
contacts restore utils

"""
import csv
from django.core.validators import validate_email, ValidationError
from django.core.exceptions import ObjectDoesNotExist

from acomea.contact.models import Contact, Email, Group


def contact_groups_restore(file_path, export_file_path=None, db_restore=False):
    email_contact_dict = _get_email_key_contact_dict(
        _get_contact_dict_list_from_file(file_path)
    )
    emails = Email.objects.filter(
        contact__user__profilo__rac__stato__in="9,6"
    )
    if export_file_path:
        export_file = open(export_file_path, "w")
        message_list = []
    else:
        export_file = None
    counter = 0
    for e in emails:
        if e.email in email_contact_dict:
            contact_dict = email_contact_dict.get(e.email)
            contact = e.contact
            import_group_ids = [
                g.strip() for g in contact_dict.get(
                    'get_string_group_ids'
                ).split(',')
            ]
            missing_groups = []
            for gid in import_group_ids:
                try:
                    group = Group.objects.get(id=gid)
                    if group not in contact.groups.all():
                        if db_restore: # restore group for that contact
                            contact.groups.add(group)  # signal "contact_group_signal" change "first_group" field, to use contact instace remember to reload
                        missing_groups.append("%s" % group)
                except ObjectDoesNotExist:
                    continue
            if len(missing_groups) > 0:
                msg = "%s -> groups missing: %s" % (
                    contact, ", ".join(missing_groups)
                )
                if db_restore:
                    msg += " restored"
                if export_file:
                    message_list.append(msg+"\n")
                else:
                    print msg
                counter += 1
    if export_file:
        message_list.append("\n\n")
        message_list.append('totale: %d contatti' % counter)
        export_file.writelines(message_list)
        export_file.close()
    else:
        print '\ntotale: %d contatti' % counter


def _get_contact_dict_list_from_file(file_path):
    try:
        head_line = open(file_path, 'r').readlines()[0].split(";")
        contacts_dict_list = []
        head_line.pop(-1)
        reader = csv.reader(open(file_path), delimiter=";")
        reader.next()
        for line in reader:
            if len(line)==38:
                contacts_dict_list.append(dict(zip(head_line, line)))
        return contacts_dict_list
    except Exception, e:
        print 'error: %' % e.message
        return None


def _get_email_key_contact_dict(contact_dict_list):
    email_key_dict = {}
    for contact_dict in contact_dict_list:
        for key in ('get_default_email','get_email1', 'get_email2'):
            if key in contact_dict:
                email = contact_dict.get(key)
                try:
                    validate_email(email)
                    email_key_dict[email] = contact_dict
                    break
                except ValidationError:
                    continue
                except KeyError:
                    print 'key gia presente: %s' % email
                    return
    return email_key_dict